// $Id: PacketCaptureBase.java,v 1.2 2008/08/18 05:34:36 esmondpitt Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.capture;
import java.nio.ByteBuffer;
import net.sourceforge.jpcap.util.Timeval;
import net.sourceforge.jpcap.net.Packet;
import net.sourceforge.jpcap.net.RawPacket;
import net.sourceforge.jpcap.net.PacketFactory;


/**
 * This class contains base functionality shared by 
 * both the simulator and the actual packet capture system.
 * <p>
 * Regardless of whether 'real' packets are being captured off a network 
 * or 'fake' packets are being generated by the simulator, 
 * the packet dispatch logic is the same.
 * <p>
 * This class is abstract because it is useless standalone. The 
 * packet capture system which inherits from this class must provide a 
 * way of feeding data to handlePacket() where it is then dispatched to 
 * listeners.
 *
 * @author Patrick Charles and Jonas Lehmann
 * @author Esmond Pitt
 * @version $Revision: 1.2 $
 * @lastModifiedBy $Author: esmondpitt $
 * @lastModifiedAt $Date: 2008/08/18 05:34:36 $
 */
public abstract class PacketCaptureBase 
  extends PacketDispatcher
{
  // internal packet handling implementation

  /**
   * Handle arriving packets.
   * <p>
   * Arriving packets are dispatched to registered packet listeners.
   * If caplen is smaller than length, then the packet was truncated 
   * because the amount of data on the wire exceeded the snapshot length
   * specified when open() was called.
   * <p>
   * In the simulator case, fake packets are generated and this method 
   * called with the fabricated data. In the case of the capture system,
   * the native library libjpcap calls the method back.
   *
   * @param length the length of the packet off of the wire.
   * @param caplen the number of bytes actually captured. 
   * @param seconds the seconds component of the timestamp.
   * @param useconds the microseconds component of the timestamp.
   * @param dbb the contents of the captured packet in a direct ByteBuffer:
   * note that the direct content must be copied out immediately,
   * because it is only valid for the duration of this callback.
   */
  protected synchronized void handlePacket(int length, int caplen, 
                           int seconds, int useconds, ByteBuffer dbb) {

//	  System.out.println(this.getClass().getName()+".handlePacket: received packet length="+length+" caplen="+caplen+" bb="+bb);
	  
	  if (getObjListenerCount() + getRawListenerCount() == 0)
		  return;
	  
	  // create a raw packet and abstract packet from the raw data
	  Timeval tv = new Timeval(seconds, useconds);
//	  System.out.println(this.getClass().getName()+"."handlePacket: received packet "+new BigInteger(data).toString(16));
	  
	  if (getRawListenerCount() > 0)
	  {
		  byte[]	data = new byte[caplen];
		  assert caplen > 0 : "packet caplen is zero";
		  assert dbb.limit() > 0 : "ByteBuffer is empty";
		  assert dbb.limit() >= caplen : "ByteBuffer does not contain "+caplen+" bytes";

		  dbb.position(0);
		  dbb.get(data);
		  RawPacket rp = new RawPacket(tv, data,
			  length > caplen ? length - caplen : 0);
		  
		  // dispatch the raw packet to registered listeners..
		  dispatchRawPacket(rp);
	  }
	  
	  if (getObjListenerCount() > 0)
	  {
		  // Copy the transient direct data into a non-direct ByteBuffer
		  // so the Packet constructed is of permanent use.
		  // This might seem like a performance hit, but
		  // (a) the packets are only 'caplen' long, often 64 bytes or less
		  // (b) the data has to be at least partially parsed, and therefore retrieved anyway,
		  // in order for the PacketFactory to construct the correct packet type,
		  // so much of the data is about to cross the JNI/Java boundary anyway.
		  ByteBuffer	bb = ByteBuffer.allocate(dbb.limit());
		  dbb.position(0);
		  bb.put(dbb);
		  Packet p = PacketFactory.dataToPacket(linkType, bb, tv);
		  
		  // dispatch the packet to listeners interested in packet objects..
		  dispatchPacket(p);
	  }
  }

  /**
   * The link-type code for the currently open network device.
   * The type is queried and set in open() and then utilized when 
   * packets are received by handlePacket().
   */
  protected int linkType;

  private static final String _rcsid = 
    "$Id: PacketCaptureBase.java,v 1.2 2008/08/18 05:34:36 esmondpitt Exp $";
}
