// $Id: PacketCaptureCapable.java,v 1.3 2008/09/30 22:45:51 esmondpitt Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.capture;
import java.io.Closeable;


/**
 * This is the packet capture interface.
 * It is implemented by both the packet capture system {@link PacketCapture}
 * and the simulator {@link net.sourceforge.jpcap.simulator.PacketCaptureSimulator}.
 * <p>
 * The interface has two major components:
 * <ul>
 * <li> methods that a client calls in order to setup and initiate packet capture.
 * <li> methods that a client uses to register for packet events
 *</ul>
 *
 * @author Patrick Charles and Jonas Lehmann
 * @author Esmond Pitt
 * @version $Revision: 1.3 $
 * @lastModifiedBy $Author: esmondpitt $
 * @lastModifiedAt $Date: 2008/09/30 22:45:51 $
 */
public interface PacketCaptureCapable extends Closeable
{
  /**
   * Snapshot length. Maximum number of bytes per packet to capture.
   * For IPv4, 96 bytes guarantees that at least the headers of most 
   * packet types and protocols will get captured.
   * For IPv6, 68 is a better value?
   */
  int DEFAULT_SNAPLEN = 96;

  /**
   * Default capture timeout in milliseconds.
   */
  int DEFAULT_TIMEOUT = 1000;

  // capture component of interface

  /**
   * Open a network device for data capture.
   *<p>
   * This call is equivalent to
   *<pre>
   *<code>
   *	open(device, DEFAULT_SNAPLEN, promiscuous, DEFAULT_TIMEOUT);
   *</code>
   *</pre>
   *
   * @param device the name of the network device. 
   * @param promiscuous whether or not the device should be put into
   * 'promiscuous' mode: true => capture all packets; false => capture packets only to and from this system.
   *
   * @see #open(String, int, boolean, int)
   */
  void open(String device, boolean promiscuous)
    throws CaptureDeviceOpenException;

  /**
   * Open a network device for data capture.
   * 
   * @param device the name of the network device. 
   * Examples of valid network devices on linux are 'eth0' and 'ppp0'.
   * @param snaplen the 'snapshot' length. Defines the maximum number of 
   * bytes to save from each captured packet.
   * @param promiscuous whether or not the device should be put into
   * 'promiscuous' mode: true => capture all packets; false => capture packets only to and from this system.
   * @param timeout the packet capture timeout in milliseconds.
   * Defines the maximum amount of time that capture will wait for another packet,
   * if the capture device's buffer isn't full,
   * before notifying the application of the packets already captured.
   * A value of zero means return each packet immediately: this mode is rather inefficient.
   * Timeouts of 500ms-2 seconds are reasonable, or longer if the application can wait longer.
   * @exception CaptureDeviceOpenException can't open the specified device
   */
  void open(String device, int snaplen, boolean promiscuous, int timeout) 
    throws CaptureDeviceOpenException;

 /**
   * Open a tcpdump-formatted savefile for reading.
   * 
   * @param fileName the name of the savefile. 
   * @exception CaptureFileOpenException can't open the specified file
   */
  void openOffline(String fileName) 
    throws CaptureFileOpenException;

  /**
   * Create, compile and activate a filter from a filter expression.
   *
   * @param filterExpression the filter expression. For example, 
   * the expression "host techno" would filter only packets sent or 
   * arriving at the host named techno.
   * @param optimize whether or not the resulting bpf code is optimized 
   * internally by libpcap.
   * @exception InvalidFilterException invalid filter specification
   */
  void setFilter(String filterExpression, boolean optimize)
    throws InvalidFilterException;

  /**
   * Capture packets.
   *
   * @param count the number of packets to capture.
   * If count is -1, this method will run forever unless {@link #endCapture} is called
   * or an exception is thrown.
   * @exception CapturePacketException Capture problem
   */
  void capture(int count)
    throws CapturePacketException;

  /**
   * Stop capturing packets. Any current {@link #capture} call will return.
   * Does not guarantee that no further packets will be processed after it is called; one more packet might be processed.
   */
  void endCapture();

  /**
   * Fetch statistics on captured packets.
   * This method should not be called unless {@link #capture} was previously called,
   * with capture occurring from a network device.
   *
   * @return packet capture statistics, or null if capture was from a file.
   */
  CaptureStatistics getStatistics();

  /**
   * {@inheritDoc}
   *
   * Close cleans up after a packet capture session.
   * It does <i>not</i> terminate a packet capture. 
   * {@link #capture} does not return control until 'count' packets are captured.
   * <p>
   * To signal an end to a capture session before the specified number of packets has been received,
   * first call {@link #endCapture}.
   */
  void close();


  // Methods to fetch capture device and network information

  /**
   * Detect a network device suitable for packet capture.
   * <p>
   * This API should be static; does not work reliably; and will be removed in a future release.
   *
   * @deprecated Please use {@link PacketCapture#lookupCaptureDevices}.
   *
   * @return the network device name.
   *
   * @exception CaptureDeviceNotFoundException No capture devices found
   */
  String findDevice() 
    throws CaptureDeviceNotFoundException;

  /**
   * Fetch the network address for the specified device.
   * <p>
   * This API:
   * <ul>
   * <li>does not work reliably, the byte-ordering being uncertain
   * <li>cannot by design cope with interfaces that have multiple IP addresses
   * <li>cannot by design cope with IPv6 addresses.
   *</ul>
   *<p>
   * For these reasons it will be removed in a future release.
   *</ul>
   * @deprecated Please use {@link PacketCapture#lookupCaptureDevices}.
   *
   * @param device the name of the network device. 
   * @return the network address
   * @exception CaptureConfigurationException Capture device not found
   */
  int getNetwork(String device)
    throws CaptureConfigurationException;

  /**
   * Fetch the network mask for the specified device.
   * <p>
   * This API:
   * <ul>
   * <li>does not work reliably, the byte-ordering being uncertain
   * <li>cannot by design cope with interfaces that have multiple IP addresses
   * <li>cannot by design cope with IPv6 addresses.
   *</ul>
   *<p>
   * For these reasons it will be removed in a future release.
   *
   * @deprecated Please use {@link PacketCapture#lookupCaptureDevices}.
   *
   * @param device the name of the network device. 
   * @return the netmask address
   * @exception CaptureConfigurationException Capture device not found
   */
  int getNetmask(String device)
    throws CaptureConfigurationException;

  /**
   * Return a description of the link layer for the current capture device,
   * corresponding to {@link #getLinkLayerType()}.
   *
   * @return link layer description.
   * @exception CaptureConfigurationException Unknown link layer type.
   *
   * @since 2.0.0
   */
  String	getLinkLayerDescription()
    throws CaptureConfigurationException;

	/**
   * Fetch the link layer type for the current capture device.
   *
   * @return the link layer type code.
   * @exception CaptureConfigurationException Capture configuration lookup problem
   */
  int getLinkLayerType()
    throws CaptureConfigurationException;

  /**
   * Get the snapshot length specified when this network device was opened.
   *
   * @return the packet snapshot length, or -1 if a capture device is not open.
   */
  int getSnapshotLength();

  // listener registration component of interface

  /**
   * Register a raw packet listener with this capture system.
   * This method is safe to call during a packet-arrival callback,
   * e.g. to remove the current listener,
   * but won't take effect until the next packet in that case.
   * @param listener the raw packet listener to add to the notification 
   * list.
   * @exception IllegalArgumentException 'listener' is null
   */
  void addRawPacketListener(RawPacketListener listener);

  /**
   * Deregister a raw packet listener from this capture system.
   * This method is safe to call during a packet-arrival callback,
   * e.g. to remove the current listener,
   * but won't take effect until the next packet in that case.
   * @param listener the raw packet listener to remove from the 
   * notification list.
   * @exception IllegalArgumentException 'listener' not registered
   */
  void removeRawPacketListener(RawPacketListener listener);

  /**
   * Register a packet object listener with this capture system.
   * This method is safe to call during a packet-arrival callback,
   * e.g. to remove the current listener,
   * but won't take effect until the next packet in that case.
   * @param listener the packet listener to add to the notification list.
   * @exception IllegalArgumentException 'listener' is null
   */
  void addPacketListener(PacketListener listener);

  /**
   * Deregister a packet object listener from this capture system.
   * This method is safe to call during a packet-arrival callback,
   * e.g. to remove the current listener,
   * but won't take effect until the next packet in that case.
   * @param listener the packet listener to remove from the 
   * notification list.
   * @exception IllegalArgumentException 'listener' not registered
   */
  void removePacketListener(PacketListener listener);
}
