/*
 * DTP.java
 *
 * Created on 30 April 2008, 15:55
 *
 ***************************************************************************
 * Copyright (C) 2008, Esmond Pitt                                         *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************
 */

package net.sourceforge.jpcap.net;

import java.nio.ByteBuffer;
import net.sourceforge.jpcap.util.AnsiEscapeSequences;
import net.sourceforge.jpcap.util.Timeval;

/**
 * DTP - Dynamic Trunking Protocol
 *
 * @since 2.0.0
 *
 * @author Esmond Pitt
 * @version $Revision: 1.1 $
 */
public interface DTP extends SNAP
{
	
	/**
	 * Offset of DTP header
	 */
	public static final int DTP_HEADER_POS = SNAP_HEADER_POS+SNAP_HEADER_LEN;
	
	/**
	 * Offset of DTP version
	 */
	public static final int DTP_VERSION_POS = DTP_HEADER_POS+0;
	
	/**
	 * Length of DTP version
	 */
	public static final int DTP_VERSION_LEN = 1;
	
	/**
	 * Length of DTP header
	 */
	public static final int DTP_HEADER_LEN = DTP_VERSION_LEN;

	/**
	 * DTP packet class.
	 *
	 * @since 2.0.0
	 *
	 * @author Esmond Pitt
	 * @version $Revision: 1.1 $
	 */
	class Packet extends EthernetPacket
	{
		static final long serialVersionUID = -1388565391689826930L;

		/**
		 * Creates a new DTP packet
		 */
		public Packet(int lLen, byte [] bytes)
		{
			this(lLen, bytes, null);
		}
		
		/**
		 * Create a new DTP packet.
		 */
		public Packet(int lLen, byte [] bytes, Timeval tv)
		{
			this(lLen, ByteBuffer.wrap(bytes), tv);
		}
		
		/**
		 * Create a new DTP packet
		 * @param lLen Packet length
		 * @param bb Packet bytes in a ByteBuffer
		 * @param tv TimeValue
		 */
		public Packet(int lLen, ByteBuffer bb, Timeval tv)
		{
			super(lLen, bb, tv);
		}
		
		public int	getDTPVersion()
		{
			return _byteBuffer.get(_ethOffset+DTP_VERSION_POS);
		}
		
		/**
		 * Set the DTP protocol version
		 * @param version DTP protocol version
		 */
		public void	setDTPVersion(int version)
		{
			_byteBuffer.put(_ethOffset+DTP_VERSION_POS, (byte)version);
		}
		
		public byte[]	getData()
		{
			return getDTPData();
		}
		
		/**
		 * @return a <i>copy</i> of the DTP data, which is a series of MIBs in
		 * the format type:length:value, where 'type' and 'length' are two bytes each and 'length' includes
		 * the length of 'type' and 'length': as the data is a copy, changing it has no effect on the DTP packet object.
		 */
		public byte[]	getDTPData()
		{
			_byteBuffer.position(_ethOffset+DTP_HEADER_POS+DTP_HEADER_LEN);
			 byte[]	data = new byte[_byteBuffer.remaining()];
			 _byteBuffer.get(data);
			 return data;
		}
		
		/**
		 * Set the DTP data payload.
		 * @param data Payload data.
		 */
		public void	setDTPData(byte[] data)
		{
			_byteBuffer.position(_ethOffset+DTP_HEADER_LEN);
			_byteBuffer.put(data);
			_byteBuffer.limit(_byteBuffer.position());
		}
		
		public String	toColoredString(boolean colored)
		{
			StringBuffer buffer = new StringBuffer();
			buffer.append('[');
			if(colored) buffer.append(getColor());
			buffer.append("DTPPacket");
			if(colored) buffer.append(AnsiEscapeSequences.RESET);
			buffer.append(": ");
			buffer.append(getSourceMACAddress().toString() + "->" +
				getDestinationMACAddress().toString());
			buffer.append(", v="+getDTPVersion());
			buffer.append(" l="+DTP_HEADER_LEN+","+getDTPData().length);
			buffer.append(']');
			return buffer.toString();
			
		}
	}
}
