// $Id: IGMPPacket.java,v 1.2 2008/08/18 05:34:37 esmondpitt Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.net;

import java.net.InetAddress;
import java.net.UnknownHostException;
import net.sourceforge.jpcap.util.AnsiEscapeSequences;
import net.sourceforge.jpcap.util.Timeval;
import java.nio.ByteBuffer;


/**
 * An IGMP packet.
 * <p>
 * Extends an IP packet, adding an IGMP header and IGMP data payload.
 *
 * @author Patrick Charles
 * @author Jonas Lehmann
 * @author Esmond Pitt
 * @version $Revision: 1.2 $
 * @lastModifiedBy $Author: esmondpitt $
 * @lastModifiedAt $Date: 2008/08/18 05:34:37 $
 */
public class IGMPPacket extends IPPacket implements IGMPFields, IPv4.IGMP
{
	static final long serialVersionUID = -5000841281390612718L;
	
 /**
   * Construct a new IGMP packet.
   *
   * @param lLen link-layer header length
   * @param bytes Packet data
   */
  public IGMPPacket(int lLen, byte [] bytes) {
    this(lLen, bytes, null);
  }

  /**
   * Construct a new IGMP packet, including the capture time.
   *
   * @param lLen link-layer header length
   * @param bytes Packet data
   * @param tv Timestamp
   */
  public IGMPPacket(int lLen, byte [] bytes, Timeval tv) {
    super(lLen, bytes, tv);
  }

  /**
   * Construct a new IGMP packet, including the capture time.
   *
   * @param lLen link-layer header length
   * @param bb Packet data in a ByteBuffer
   * @param tv Timestamp
   * @since 2.0.0
   */
  public IGMPPacket(int lLen, ByteBuffer bb, Timeval tv) {
    super(lLen, bb, tv);
  }

  public byte[] getIGMPHeader() {
	byte[]	bytes = new byte[IGMP_HEADER_LEN];
	_byteBuffer.position(_ipOffset);
	_byteBuffer.get(bytes);
	return bytes;
  }

  /**
   * {@inheritDoc}
   */
  public byte[] getHeader() {
    return getIGMPHeader();
  }

  public byte[] getIGMPData() {
	_byteBuffer.position(_ipOffset+IGMP_HEADER_LEN);
 	byte[]	bytes = new byte[_byteBuffer.remaining()];
	_byteBuffer.get(bytes);
	return bytes;
  }

  /**
   * {@inheritDoc}
   */
  public byte[] getData() {
    return getIGMPData();
  }

  public int getMessageType() {
	return _byteBuffer.get(_ipOffset+IGMP_CODE_POS) & 0xff;
  }
  
  public void	setMessageType(byte type)
  {
	  _byteBuffer.put(_ipOffset+IGMP_CODE_POS, type);
  }

  public int getMaxResponseTime() {
	return _byteBuffer.get(_ipOffset+IGMP_MRSP_LEN) & 0xff;
  }

  public void setMaxResponseTime(byte time) {
	_byteBuffer.put(_ipOffset+IGMP_MRSP_POS, time);
  }

  public int getIGMPChecksum() {
    return _byteBuffer.getShort(_ipOffset+IGMP_CSUM_POS) & 0xffff;
  }

  /**
   * {@inheritDoc}
   */
  public int getChecksum() {
    return getIGMPChecksum();
  }

  /**
   * Fetch the IGMP group address.
   * @return the IGMP group address.
   * @deprecated Please use getGroupInetAddress().
   */
  @Deprecated
  public String getGroupAddress() {
	  return getGroupInetAddress().getHostAddress();
  }

  public InetAddress getGroupInetAddress()
  {
	byte[]	bytes = new byte[IPAddress.WIDTH];
	_byteBuffer.position(_ipOffset+IGMP_GADDR_POS);
	_byteBuffer.get(bytes);
	try
	{
	    return InetAddress.getByAddress(bytes);
	}
	catch (UnknownHostException exc)
	{
		// Unexpected.
		exc.printStackTrace();
		return null;
	}
  }

  public void setGroupInetAddress(InetAddress address)
  {
	_byteBuffer.position(_ipOffset+IGMP_GADDR_POS);
	_byteBuffer.put(address.getAddress());
  }

  /**
   * {@inheritDoc}
   */
  public boolean	isValidChecksum()
  {
	  return super.isValidChecksum()
	  && getIGMPChecksum() == computeSenderChecksum(_ipOffset, IGMP_HEADER_LEN+getIGMPData().length, IGMP_CSUM_POS);
  }

  /**
   * {@inheritDoc}
   */
  public void	setChecksums()
  {
	  int	checksum = computeSenderChecksum(_ipOffset, IGMP_HEADER_LEN+getIGMPData().length, IGMP_CSUM_POS);
	  // Forget cache. NB getIGMPHeader() re-establishes it so can't be called after this.
	  _byteBuffer.putShort(_ipOffset+IGMP_CSUM_POS, (short)checksum);
	  super.setChecksums();
  }

  /**
   * {@inheritDoc}
   */
  public String toColoredString(boolean colored) {
    StringBuffer buffer = new StringBuffer();
    buffer.append('[');
    if(colored) buffer.append(getColor());
    buffer.append("IGMPPacket");
    if(colored) buffer.append(AnsiEscapeSequences.RESET);
    buffer.append(": ");
    buffer.append(IGMPMessage.getDescription(getMessageType()));
    buffer.append(", ");
    buffer.append(getGroupAddress() + ": ");
    buffer.append(getSourceInetAddress().getHostAddress() + "->" + 
                  getDestinationInetAddress().getHostAddress());
    buffer.append(" l=" + IGMP_HEADER_LEN + "," + 
		  (_byteBuffer.limit()-_ipOffset-IGMP_HEADER_LEN));
    buffer.append(']');

    return buffer.toString();
  }

  /**
   * {@inheritDoc}
   */
  public String getColor() {
    return AnsiEscapeSequences.BROWN;
  }


  private static final String _rcsid = 
    "$Id: IGMPPacket.java,v 1.2 2008/08/18 05:34:37 esmondpitt Exp $";
}
