/*
 * IPv4.java
 *
 * Created on 30 May 2008, 12:31
 *
 * Copyright (c) Esmond Pitt, 2007. All rights reserved.
 */

package net.sourceforge.jpcap.net;

import java.net.Inet4Address;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.BufferUnderflowException;
import java.nio.ByteBuffer;
import net.sourceforge.jpcap.util.AnsiEscapeSequences;
import net.sourceforge.jpcap.util.Timeval;

/**
 * IPv4 interface.
 *
 * @author Esmond Pitt
 * @version $Revision: 1.3 $
 *
 * @since 2.0.0
 */
public interface IPv4 extends IPFields
{
	int	IPv4_VERSION = 0x04;
	/**
	 * Base IPv4 packet class.
	 *
	 * @author Esmond Pitt
	 * @version $Revision: 1.3 $
	 *
	 * @since 2.0.0.
	 */
	abstract class Packet extends IP.Packet implements IPFields
	{
		static final long serialVersionUID = -3910395791298094043L;
		
		public Packet(int lLen, byte[] bytes)
		{
			this(lLen, bytes, null);
			assert getVersion() == IPv4_VERSION : "Incorrect IPv4 version "+getVersion();
		}
		
		public Packet(int lLen, byte[] bytes, Timeval tv)
		{
			this(lLen, ByteBuffer.wrap(bytes), tv);
		}

		public Packet(int lLen, ByteBuffer bb, Timeval tv)
		{
			super(lLen, bb, tv);
		}
		
		public int getVersion()
		{
			return (_byteBuffer.get(_ethOffset+IP_VER_POS) >> 4) & 0xf;
		}
		
		public int getIPHeaderLength()
		{
			return Math.min((_byteBuffer.get(_ethOffset+IP_VER_POS) & 0xf) * 4, _byteBuffer.limit()-_ethOffset);
		}

		/**
		 * Fetch the type of service.
		 * For more information refer to the {@link TypesOfService} interface.
		 * @return the type of service.
		 * @see TypesOfService
		 */
		public int getTypeOfService()
		{
//	return ArrayHelper.extractInteger(_bytes, _ethOffset + IP_TOS_POS, IP_TOS_LEN);
			return _byteBuffer.get(_ethOffset+IP_TOS_POS) & 0xff;
		}
		
		public int getLength()
		{
//    return ArrayHelper.extractInteger(_bytes, _ethOffset + IP_LEN_POS, IP_LEN_LEN);
			return _byteBuffer.getShort(_ethOffset+IP_LEN_POS) & 0xffff;
		}
		
		public void setLength(int length)
		{
			_byteBuffer.position(_ethOffset+IP_LEN_POS);
			 _byteBuffer.putShort((short)length);
		}

		/**
		 * Fetch the unique ID of this IP packet. The ID normally
		 * increments by one each time a packet is sent by a host.
		 * @return the unique packet ID
		 */
		public int getId()
		{
//    return ArrayHelper.extractInteger(_bytes, _ethOffset + IP_ID_POS, IP_ID_LEN);
			return _byteBuffer.getShort(_ethOffset+IP_ID_POS) & 0xffff;
		}
		
		/**
		 * Fetch fragmentation flags.
		 * @return the fragmentation flags
		 */
		public int getFragmentFlags()
		{
			// fragment flags are the high 3 bits
//      return (ArrayHelper.extractInteger(_bytes, _ethOffset + IP_FRAG_POS, IP_FRAG_LEN) >> 13) & 0x7;
			return (_byteBuffer.getShort(_ethOffset+IP_FRAG_POS) >> 13) & 0x7;
		}
		
		/**
		 * Fetch fragment offset.
		 * @return the fragment offset
		 */
		public int getFragmentOffset()
		{
			// offset is the low 13 bits
//    return ArrayHelper.extractInteger(_bytes, _ethOffset + IP_FRAG_POS, IP_FRAG_LEN) & 0x1fff;
			return _byteBuffer.getShort(_ethOffset+IP_FRAG_POS) & 0x1fff;
		}
		
		/**
		 * Fetch the time to live. TTL sets the upper limit on the number of
		 * routers through which this IP datagram is allowed to pass.
		 * @return the TTL
		 */
		public int getTimeToLive()
		{
//    return ArrayHelper.extractInteger(_bytes, _ethOffset + IP_TTL_POS, IP_TTL_LEN);
			return _byteBuffer.get(_ethOffset+IP_TTL_POS) & 0xff;
		}
		
		public int getIPProtocol()
		{
//    return ArrayHelper.extractInteger(_bytes, _ethOffset + IP_CODE_POS, IP_CODE_LEN);
			return _byteBuffer.get(_ethOffset+IP_CODE_POS) & 0xff;
		}
  
		/**
		 * {@inheritDoc}
		 */
		public int getProtocol()
		{
			return getIPProtocol();
		}
		
		/**
		 * Fetch the IP checksum.
		 * @return the IP checksum
		 */
		public int getIPChecksum()
		{
//    return ArrayHelper.extractInteger(_bytes, _ethOffset + IP_CSUM_POS, IP_CSUM_LEN);
			return _byteBuffer.getShort(_ethOffset + IP_CSUM_POS) & 0xffff;
		}
		
		/**
		 * {@inheritDoc}
		 */
		@Override
			public int getChecksum()
		{
			return getIPChecksum();
		}
		
		public InetAddress getSourceInetAddress()
		{
			try
			{
				byte[]	address = new byte[IPAddress.WIDTH];
				_byteBuffer.position(_ethOffset+IP_SRC_POS);
				_byteBuffer.get(address);
				return InetAddress.getByAddress(address);
			}
			catch (UnknownHostException exc)
			{
				// Unexpected
				exc.printStackTrace();
				return null;
			}
		}
		
		public void	setSourceInetAddress(InetAddress address)
		{
			if (!(address instanceof Inet4Address))
				throw new IllegalArgumentException("cannot assign IPv6 address to IPv4 packet");
			_byteBuffer.position(_ethOffset + IP_SRC_POS);
			_byteBuffer.put(address.getAddress());
		}
		
		public InetAddress getDestinationInetAddress()
		{
			try
			{
				byte[]	address = new byte[IPAddress.WIDTH];
				_byteBuffer.position(_ethOffset+IP_DST_POS);
				_byteBuffer.get(address);
				return InetAddress.getByAddress(address);
			}
			catch (UnknownHostException exc)
			{
				// Unexpected
				exc.printStackTrace();
				return null;
			}
		}
		
		public void	setDestinationInetAddress(InetAddress address)
		{
			if (!(address instanceof Inet4Address))
				throw new IllegalArgumentException("cannot assign IPv6 address to IPv4 packet");
			_byteBuffer.position(_ethOffset + IP_DST_POS);
			_byteBuffer.put(address.getAddress());
		}
		
		public byte[] getIPHeader()
		{
//      return PacketEncoding.extractHeader(_ethOffset, getIPHeaderLength(), _bytes);
			byte[]	bytes = new byte[getIPHeaderLength()];
			_byteBuffer.position(_ethOffset);
			_byteBuffer.get(bytes);
			return bytes;
		}
		
		public byte[] getIPData()
		{
			_byteBuffer.position(_ethOffset+getIPHeaderLength());		
			// We need to use the IP lengths given in the packet to avoid any ethernet trailer bits.
			// We should probably fix this so they are stripped at the ethernet packet level
			// but that would be to much work.  :-)
			byte[]	bytes = new byte[getLength()-getIPHeaderLength()]; 
			_byteBuffer.get(bytes);
			return bytes;
		}
		
		/**
		 * {@inheritDoc}
		 */
		public boolean isValidChecksum()
		{
			return super.isValidChecksum() && isValidIPChecksum();
		}
		
		/**
		 * Check if the IP packet is valid, checksum-wise.
		 * @return true iff the IP checksum is valid.
		 */
		public boolean isValidIPChecksum()
		{
			// first validate other information about the packet. if this stuff
			// is not true, the packet (and therefore the checksum) is invalid
			// - ip_hl >= 5 (ip_hl is the length in 4-byte words)
			if( getIPHeaderLength() < IP_HEADER_LEN )
				return false;
			return computeReceiverIPChecksum() == 0;
		}
		
		protected int computeReceiverIPChecksum()
		{
			return computeReceiverChecksum(_ethOffset, getIPHeaderLength());
		}
		protected int computeReceiverChecksum(int start, int len)
		{
			// checksum should come out to zero if checksum is correct
			byte[]	bytes = new byte[len];
			_byteBuffer.position(start);
			_byteBuffer.get(bytes);
			return ipChecksum(bytes,0,len);
		}
		
		protected int computeSenderIPChecksum()
		{
			return computeSenderChecksum(_ethOffset, getIPHeaderLength(), IP_CSUM_POS);
		}
		protected int computeSenderChecksum(int start, int len, int csumPos)
		{
			// quick bad-data check
			if( csumPos >= len) return 0; // bad data, header too short
			// copy bytes, zero out checksum
			byte[] sbytes = new byte[len];
			_byteBuffer.position(start);
			_byteBuffer.get(sbytes);
			// zero out any current checksum
			sbytes[csumPos] = sbytes[csumPos+1] = 0;
			// checksum should come out to zero if checksum is correct
			return ipChecksum(sbytes,0,len);
		}

		/**
		 * {@inheritDoc}
		 */
		@Override
			public void	setChecksums()
		{
			_byteBuffer.putShort(_ethOffset+IP_CSUM_POS, (short)computeSenderIPChecksum());
			super.setChecksums();
		}
		
		public String toColoredVerboseString(boolean colored)
		{
			StringBuffer buffer = new StringBuffer();
			buffer.append('[');
			if(colored) buffer.append(getColor());
			buffer.append("IPv"+getVersion()+"Packet");
			if(colored) buffer.append(AnsiEscapeSequences.RESET);
			buffer.append(": ");
			buffer.append("version=" + getVersion() + ", ");
			buffer.append("hlen=" + getHeaderLength() + ", ");
			buffer.append("tos=" + getTypeOfService() + ", ");
			buffer.append("length=" + getLength() + ", ");
			buffer.append("id=" + getId() + ", ");
			buffer.append("flags=0x" + Integer.toHexString(getFragmentFlags()) + ", ");
			buffer.append("offset=" + getFragmentOffset() + ", ");
			buffer.append("ttl=" + getTimeToLive() + ", ");
			buffer.append("proto=" + getProtocol() + ", ");
			buffer.append("sum=0x" + Integer.toHexString(getChecksum()) + ", ");
			buffer.append("src=" + getSourceInetAddress().getHostAddress() + ", ");
			buffer.append("dest=" + getDestinationInetAddress().getHostAddress());
			buffer.append(']');
			
			return buffer.toString();
		}
	}
	
	/**
	 * IPv4 PacketFactory class.
	 * @since 2.0.0
	 *
	 * @author Esmond Pitt
	 * @version $Revision: 1.3 $
	 */
	class PacketFactory
	{
		// Private ctor, no instantiation please.
		private PacketFactory(){}
		
		/**
		 * Create an IPv4 packet.
		 * @param lLen Link-layer header length
		 * @param bb ByteBuffer containing packet data
		 * @param tv Timeval of packet
		 * @return an IPv4 Packet of the appropriate class: ICMP, IGMP, TCP, UDP, or IPPacket by default.
		 */
		public static Packet	dataToPacket(int lLen, ByteBuffer bb, Timeval tv)
		{
			int	ipProtocol = bb.get(lLen + IPFields.IP_CODE_POS) & 0xff;
			switch (ipProtocol)
			{
				// ICMP
				case IPProtocols.ICMP:
					return new ICMPPacket(lLen, bb, tv);
					// IGMP
				case IPProtocols.IGMP:
					return new IGMPPacket(lLen, bb, tv);
					// TCP
				case IPProtocols.TCP:
					return new TCPPacket(lLen, bb, tv);
					// UDP
				case IPProtocols.UDP:
					return UDP.PacketFactory.dataToPacket(lLen, bb, tv);
				default:
					// unidentified ip.
					return new IPPacket(lLen, bb, tv);
			}
		}
	}
	
	/**
	 * IGMP (Internet Group Management Protocol) API interface.
	 * This supports IGMPv2 and IGMPv3.
	 * Note that IGMPv1 is obsolete and is not supported.
	 * Note also that IGMP is only used by IPv6: IPv6 uses MLD, a sub-protocol of ICMPv6.
	 *
	 * @author Esmond Pitt
	 * @version $Revision: 1.3 $
	 * 
	 * @see <a href="ftp://ftp.rfc-editor.org/in-notes/rfc2236.txt">RFC2236 - IGMPv2</a>
	 * @see <a href="ftp://ftp.rfc-editor.org/in-notes/rfc3376.txt">RFC3376 - IGMPv3</a>
	 *
	 * @since 2.0.0
	 */
	interface IGMP
	{
		/**
		 * Fetch the IGMP header as a byte array.
		 * @return the IGMP header as a byte array.
		 */
		byte[] getIGMPHeader();
		/**
		 * Fetch the IGMP data as a byte array.
		 * @return the IGMP data as a byte array.
		 */
		byte[] getIGMPData();
		/**
		 * Fetch the IGMP message type.
		 * Return value can be used with {@link IGMPMessage#getDescription}.
		 * @return a 1-byte value containing the message type.
		 */
		int getMessageType();
		/**
		 * Set the IGMP message type, as per {@link #getMessageType}.
		 * @param type message type.
		 * @see IGMPMessages
		 * @since 2.0.0.
		 */
		void	setMessageType(byte type);
		/**
		 * Fetch the IGMP max response time.
		 * @return the IGMP max response time, in tenths of a second.
		 */
		int getMaxResponseTime();
		/**
		 * Set the IGMP max response time.
		 * @param time the IGMP max response time, in tenths of a second.
		 */
		void	setMaxResponseTime(byte time);
		/**
		 * Fetch the IGMP checksum.
		 * @return the IGMP checksum.
		 * @since 2.0.0.
		 */
		int getIGMPChecksum();
		/**
		 * Fetch the IGMP group address.
		 * @return the IGMP group address.
		 * @since 2.0.0
		 */
		InetAddress getGroupInetAddress();
		/**
		 * Set the IGMP group address.
		 * @param address Group address.
		 * @since 2.0.0.
		 */
		void	setGroupInetAddress(InetAddress address);
	}
	
	/**
	 * TCP/IPv4 interface.
	 *
	 * @author Esmond Pitt
	 * @version $Revision: 1.3 $
	 *
	 * @since 2.0.0
	 */
	interface TCP extends IP.TCP, IPv4
	{
		static final long serialVersionUID = 7722034053042394219L;

		/**
		 * TCP/IPv4 packet class.
		 *
		 * @author Esmond Pitt
		 * @version $Revision: 1.3 $
		 *
		 * @since 2.0.0
		 */
		class Packet extends IPv4.Packet implements TCP, TCPFields, IPFields
		{
			private int _tcpHeaderLength;
			
			/**
			 * Create a new TCP packet.
			 * @param lLen link-layer header length
			 * @param bb Packet data in a ByteBuffer
			 * @param tv Timestamp
			 * @since 2.0.0
			 */
			public Packet(int lLen, ByteBuffer bb, Timeval tv)
			{
				super(lLen, bb, tv);
				// set TCP header length
				try
				{
					_tcpHeaderLength = ((_byteBuffer.getShort(_ipOffset + TCP_FLAG_POS) >> 12) & 0xf) * 4;
				}
				catch (IndexOutOfBoundsException exc)
				{
					
				}
			}
			
			public int getSourcePort()
			{
				return _byteBuffer.getShort(_ipOffset + TCP_SP_POS) & 0xffff;
			}
			
			public int getDestinationPort()
			{
				return _byteBuffer.getShort(_ipOffset + TCP_DP_POS) & 0xffff;
			}
			
			public long getSequenceNumber()
			{
				return _byteBuffer.getInt(_ipOffset + TCP_SEQ_POS) & 0xffffffffL;
			}
			
			public long getAcknowledgmentNumber()
			{
				return _byteBuffer.getInt(_ipOffset + TCP_ACK_POS) & 0xffffffffL;
			}
			
			public int getTCPHeaderLength()
			{
				return _tcpHeaderLength;
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public int getHeaderLength()
			{
				return getTCPHeaderLength();
			}
			
			public int getPayloadDataLength()
			{
				// set data (payload) length based on info in headers (note: tcpdump
				//  can return extra junk bytes which bubble up to here
				int tmpLen = getLength() - getIPHeaderLength() - _tcpHeaderLength;
				return tmpLen < 0 ? 0 : tmpLen;
			}
			
			public int getWindowSize()
			{
				return _byteBuffer.getShort(_ipOffset+TCP_WIN_POS) & 0xffff;
			}
			
			public int getTCPChecksum()
			{
				return _byteBuffer.getShort(_ipOffset+TCP_CSUM_POS) & 0xffff;
			}
			
			/**
			 * {@inheritDoc}
			 */
			public int getChecksum()
			{
				return getTCPChecksum();
			}
			
			public int getUrgentPointer()
			{
				return _byteBuffer.getShort(_ipOffset+TCP_URG_POS) & 0xffff;
			}
			
			private int getAllFlags()
			{
				return _byteBuffer.getShort(_ipOffset+TCP_FLAG_POS) & 0xffff;
			}
			
			// EJP 7 May 2008: deleted ten booleans that were there to manage five bits!
			// Quicker and smaller to just test the appropriate flag bit.
			// Similarly, removed many other redundant xxxSet booleans that could be deduced
			// from an impossible value for xxx.
			
			public boolean isUrg()
			{
				return (getAllFlags() & TCP_URG_MASK) != 0;
			}
			
			public boolean isAck()
			{
				return (getAllFlags() & TCP_ACK_MASK) != 0;
			}
			
			public boolean isPsh()
			{
				return (getAllFlags() & TCP_PSH_MASK) != 0;
			}
			
			public boolean isRst()
			{
				return (getAllFlags() & TCP_RST_MASK) != 0;
			}
			
			public boolean isSyn()
			{
				return (getAllFlags() & TCP_SYN_MASK) != 0;
			}
			
			public boolean isFin()
			{
				return (getAllFlags() & TCP_FIN_MASK) != 0;
			}
			
			public byte[] getTCPHeader()
			{
				byte[]	data = new byte[getTCPHeaderLength()];
				try
				{
					_byteBuffer.position(_ipOffset);
					_byteBuffer.get(data);
				}
				catch (IllegalArgumentException exc)
				{
					
				}
				catch (BufferUnderflowException exc)
				{
					
				}
				return data;
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public byte[] getHeader()
			{
				return getTCPHeader();
			}
			
			public byte[] getTCPData()
			{
				// set data length based on info in headers (note: tcpdump
				//  can return extra junk bytes which bubble up to here
				try
				{
					_byteBuffer.position(_ipOffset+getTCPHeaderLength());
				}
				catch (IllegalArgumentException exc)
				{
				}
				catch (BufferUnderflowException exc)
				{
				}
				byte[]	data = new byte[_byteBuffer.remaining()];
				_byteBuffer.get(data);
				return data;
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public byte[] getData()
			{
				return getTCPData();
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public boolean	isValidChecksum()
			{
				return super.isValidChecksum() && isValidTCPChecksum();
			}
			
			public boolean	isValidTCPChecksum()
			{
				return getTCPChecksum() == computeSenderChecksum(_ethOffset+IP_SRC_POS, getIPHeaderLength()-IP_SRC_POS+getTCPHeader().length+getTCPData().length, TCP_CSUM_POS);
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public void	setChecksums()
			{
				// The checksum is over all the data from the IP SRC and DST addresses to the entire TCP header and data
				int checksum = computeSenderChecksum(_ethOffset+IP_SRC_POS, getIPHeaderLength()-IP_SRC_POS+getTCPHeader().length+getTCPData().length, TCP_CSUM_POS);
				_byteBuffer.putShort(_ipOffset+TCP_CSUM_POS, (short)checksum);
				super.setChecksums();
			}
			
			/**
			 * {@inheritDoc}
			 */
			public String toColoredString(boolean colored)
			{
				StringBuffer buffer = new StringBuffer();
				buffer.append('[');
				if(colored) buffer.append(getColor());
				buffer.append("TCPPacket");
				if(colored) buffer.append(AnsiEscapeSequences.RESET);
				buffer.append(": ");
				buffer.append(getSourceInetAddress().getHostAddress());
				buffer.append('.');
				buffer.append(IPPort.getName(getSourcePort()));
				buffer.append("->");
				buffer.append(getDestinationInetAddress().getHostAddress());
				buffer.append('.');
				buffer.append(IPPort.getName(getDestinationPort()));
				if(isUrg())
					buffer.append(";urg[0x" + Integer.toHexString(getUrgentPointer()) + "]");
				if(isAck())
					buffer.append(";ack[0x" +
						Long.toHexString(getAcknowledgmentNumber()) + "]");
				if(isPsh()) buffer.append(";psh");
				if(isRst()) buffer.append(";rst");
				if(isSyn())
					buffer.append(";syn[0x" +
						Long.toHexString(getSequenceNumber()) + "]");
				if(isFin()) buffer.append(";fin");
				buffer.append(" l=" + getTCPHeaderLength() + "," +
					getPayloadDataLength());
				buffer.append(']');
				
				return buffer.toString();
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public String toColoredVerboseString(boolean colored)
			{
				StringBuffer buffer = new StringBuffer();
				buffer.append('[');
				if(colored) buffer.append(getColor());
				buffer.append("TCPPacket");
				if(colored) buffer.append(AnsiEscapeSequences.RESET);
				buffer.append(": ");
				buffer.append("sport=" + getSourcePort() + ", ");
				buffer.append("dport=" + getDestinationPort() + ", ");
				buffer.append("seqn=0x" + Long.toHexString(getSequenceNumber()) + ", ");
				buffer.append("ackn=0x" +
					Long.toHexString(getAcknowledgmentNumber()) + ", ");
				buffer.append("hlen=" + getHeaderLength() + ", ");
				buffer.append("urg=" + isUrg() + ", ");
				buffer.append("ack=" + isAck() + ", ");
				buffer.append("psh=" + isPsh() + ", ");
				buffer.append("rst=" + isRst() + ", ");
				buffer.append("syn=" + isSyn() + ", ");
				buffer.append("fin=" + isFin() + ", ");
				buffer.append("wsize=" + getWindowSize() + ", ");
				buffer.append("sum=0x" + Integer.toHexString(getChecksum()) + ", ");
				buffer.append("uptr=0x" + Integer.toHexString(getUrgentPointer()));
				buffer.append(']');
				
				return buffer.toString();
			}
			
			/**
			 * {@inheritDoc}
			 */
			public String getColor()
			{
				return AnsiEscapeSequences.YELLOW;
			}
			
		}
		
	}
	
	interface UDP
	{
		class Packet extends IPv4.Packet implements UDP, UDPFields
		{
			static final long serialVersionUID = -1425875564023469577L;
			
			/**
			 * Create a new UDP packet.
			 * @param lLen link-layer header length
			 * @param bb Packet data in a ByteBuffer
			 * @param tv Timestamp
			 */
			public Packet(int lLen, ByteBuffer bb, Timeval tv)
			{
				super(lLen, bb, tv);
			}
			
			public int getSourcePort()
			{
				return _byteBuffer.getShort(_ipOffset+UDP_SP_POS) & 0xffff;
			}
			
			public int getDestinationPort()
			{
				return _byteBuffer.getShort(_ipOffset+UDP_DP_POS) & 0xffff;
			}
			
			/**
			 * {@inheritDoc}
			 *<p>
			 * Fetch the total length of the UDP packet, including header and
			 * data payload, in bytes.
			 */
			@Override
				public int getLength()
			{
				// should produce the same value as header.length + data.length
				return _byteBuffer.getShort(_ipOffset+UDP_LEN_POS) & 0xffff;
			}
			
			public int getUDPChecksum()
			{
				return _byteBuffer.getShort(_ipOffset+UDP_CSUM_POS) & 0xffff;
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public int getChecksum()
			{
				return getUDPChecksum();
			}
			
			public byte[] getUDPHeader()
			{
				byte[]	bytes = new byte[UDP_HEADER_LEN];
				_byteBuffer.position(_ipOffset);
				_byteBuffer.get(bytes);
				return bytes;
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public byte[] getHeader()
			{
				return getUDPHeader();
			}
			
			public byte[] getUDPData()
			{
				// set data length based on info in headers (note: tcpdump
				//  can return extra junk bytes which bubble up to here
				_byteBuffer.position(_ipOffset+UDP_HEADER_LEN);
				byte[]	bytes = new byte[_byteBuffer.remaining()];
				_byteBuffer.get(bytes);
				return bytes;
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public byte[] getData()
			{
				return getUDPData();
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public boolean	isValidChecksum()
			{
				return super.isValidChecksum() && isValidUDPChecksum();
			}
			
			public boolean	isValidUDPChecksum()
			{
				return getUDPChecksum() == computeSenderChecksum(_ethOffset+IP_SRC_POS, getIPHeaderLength()-IP_SRC_POS+getUDPHeader().length+getUDPData().length, UDP_CSUM_POS);
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public void	setChecksums()
			{
				// The checksum is over all the data from the IP SRC and DST addresses to the entire UDP header and data
				int	checksum = computeSenderChecksum(_ethOffset+IP_SRC_POS, getIPHeaderLength()-IP_SRC_POS+getUDPHeader().length+getUDPData().length, UDP_CSUM_POS);
				_byteBuffer.putShort(_ipOffset+UDP_CSUM_POS, (short)checksum);
				super.setChecksums();
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public String toColoredString(boolean colored)
			{
				StringBuffer buffer = new StringBuffer();
				buffer.append('[');
				if(colored) buffer.append(getColor());
				buffer.append("UDPv4Packet");
				if(colored) buffer.append(AnsiEscapeSequences.RESET);
				buffer.append(": ");
				buffer.append(getSourceInetAddress().getHostAddress());
				buffer.append('.');
				buffer.append(IPPort.getName(getSourcePort()));
				buffer.append("->");
				buffer.append(getDestinationInetAddress().getHostAddress());
				buffer.append('.');
				buffer.append(IPPort.getName(getDestinationPort()));
				buffer.append(" l=" + UDP_HEADER_LEN + "," + (getLength()-UDP_HEADER_LEN));
				buffer.append(']');
				
				return buffer.toString();
			}
			
			/**
			 * {@inheritDoc}
			 */
			@Override
				public String getColor()
			{
				return AnsiEscapeSequences.LIGHT_GREEN;
			}
		}
		
		class PacketFactory
		{
			// Private ctor, no instantiation please.
			private PacketFactory(){}

			/**
			 * Create an IPv4 packet.
			 * @param lLen Link-layer header length
			 * @param bb ByteBuffer containing packet data
			 * @param tv Timeval of packet
			 * @return an IPv4 Packet of the appropriate class: ICMP, IGMP, TCP, UDP, or IPPacket by default.
			 */
			public static IPv4.Packet	dataToPacket(int lLen, ByteBuffer bb, Timeval tv)
			{
				// TODO add DHCP etc
				// Also TODO: return new UDP.Packet(lLen, bb, tv);
				return new UDPPacket(lLen, bb, tv);
			}
		}
	}
}
