// $Id: Packet.java,v 1.3 2008/10/20 14:43:41 rickwall Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Copyright (C) 2008, Esmond Pitt                                         *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.net;

import net.sourceforge.jpcap.util.Timeval;
import java.io.Serializable;
import java.nio.ByteBuffer;


/**
 * A network packet.
 * <p>
 * This class currently contains no implementation because only Ethernet 
 * is supported. In other words, all instances of packets returned by 
 * packet factory will always be at least as specific as EthernetPacket.
 * <p>
 * On large Ethernet networks, I sometimes see packets which don't have 
 * link-level ethernet headers. If and when I figure out what these are, 
 * maybe this class will be the root node of a packet hierarchy derived 
 * from something other than Ethernet.
 *
 * @author Patrick Charles
 * @author Jonas Lehmann
 * @author Esmond Pitt
 * @version $Revision: 1.3 $
 * @lastModifiedBy $Author: rickwall $
 * @lastModifiedAt $Date: 2008/10/20 14:43:41 $
 */
public abstract class Packet implements Serializable
{
	static final long serialVersionUID = 3134677290752764541L;
	
  /**
   * Fetch the total length of the packet.
   */
  public abstract int	getPacketLength();
  
  /**
   * Convert this packet to a byte[] array.
   * The array is constructed lazily from the underlying ByteBuffer where the real
   * packet data is captured and stored.
   * As that ByteBuffer is a 'direct' ByteBuffer, the <code>toByteArray()</code> operation requires
   * copying data across the Java/JNI boundary, which is expensive.
   * The array returned is a <i>copy</i>, so altering it is futile
   * unless you construct a <i>new</i> packet with it.
   *
   * For a much more efficient read/write interface, see <code>getPacketData()</code>.  
   * @return packet data wrapped in a byte[].
   * @see #getPacketData
   */
  public abstract byte[]	toByteArray();
  
  /**
   * Fetch the packet data as a ByteBuffer.
   * This will be a 'direct' ByteBuffer, that probably doesn't support {@link java.nio.ByteBuffer#array()},
   * so its {@link java.nio.ByteBuffer#hasArray()} method will probably return false.
   * This is the form in which the packet is actually captured under the hood, and
   * processed internally, so it is the most efficient way to look at or alter packet data,
   * using all the methods of {@link java.nio.ByteBuffer}.
   *
   * For a simpler but much less efficient interface, see <code>toByteArray()</code>.  
   * @return packet data wrapped in a ByteBuffer.
   * @see #toByteArray
   */
  public abstract ByteBuffer	getPacketData();

  /**
   * Fetch header portion of the packet.
   * This is overridden in derived classes to return the header appropriate to the packet class. For example:
   *<ul>
   *<li> {@link EthernetPacket#getHeader()} returns the Ethernet header
   *<li> {@link IPPacket#getHeader()} returns the IP header, which is in the Ethernet payload
   *<li> {@link TCPPacket#getHeader()} returns the TCP header, which is in the IP payload
   *</ul>
   */
  public abstract byte [] getHeader();
  
  /**
   * Return the length of the header.
   * @return the length of the header as defined by {@link #getHeader}.
   * @see #getHeader
   */
  public abstract int	getHeaderLength();

  /**
   * Fetch the data portion of the packet.
   * This is overridden in derived classes to return successively
   * smaller and smaller amounts of data. For example:
   *<ul>
   *<li> {@link EthernetPacket#getData()} returns the entire Ethernet payload excluding the Ethernet header
   *<li> {@link IPPacket#getData()} only returns the IP payload, i.e. excluding the IP header which is in the Ethernet payload
   *<li> {@link TCPPacket#getData()} only returns the TCP payload, i.e. excluding the TCP header which is in the IP payload
   *</ul>
   */
  public abstract byte [] getData();
  
  /**
   * Return the protocol. This is overridden by derived classes: for example:
   *<ul>
   *<li> {@link EthernetPacket#getProtocol()} returns the Ethernet protocol word.
   *<li> {@link IPPacket#getProtocol()} returns the IP protocol word
   *</ul>
   * @return the protocol word.
   */
  public abstract int	getProtocol();

  /**
   * Fetch ASCII escape sequence of the color associated with this packet type.
   * @return ASCII escape sequence string
   */
  public abstract String getColor();

  /**
   * Fetch the timeval containing the time the packet arrived on the 
   * device where it was captured.
   * @return the Timeval
   */
  public abstract Timeval getTimeval();
  
  public abstract void setTimeval(Timeval tv);
  
  /**
   * Return true iff <i>all</i> the checksums in the packet are valid, otherwise false.
   * Derived classes with checksums should override, and should call <code>super.isValidChecksum()</code>
   * to check the validity of base class checksums.
   * 
   * The {@link Packet#isValidChecksum} implementation just returns true.
   * 
   * @return true iff all the checksums in the packet are valid.
   *
   * @since 2.0.0
   */
  public boolean	isValidChecksum()
  {
	  return true;
  }
  
  /**
   * Return the actual packet checksum as received.
   * Derived classes override this to return the most specific checksum, e.g.:
   *<ul>
   *<li>{@link IPPacket#getChecksum} returns the IP checksum
   *<li>{@link TCPPacket#getChecksum} returns the TCP checksum
   *</ul>
   * The implementation in {@link Packet#getChecksum} returns zero.
   *
   * @since 2.0.0
   */
  public int	getChecksum()
  {
	  return 0;
  }
  
  /**
   * Set all the checksums in this packet prior to sending.
   * Calling this method sets all the IP, ICMP, IGMP, TCP, UDP etc checksums as appropriate.
   * It is called automatically by {@link net.sourceforge.jpcap.capture.PacketOutputStream#writePacket(Packet)}.
   * Derived classes <i>must</i> override this abstract method, and <i>must</i> call <code>super.setChecksums()</code>,
   * <i>after</i> setting their own checksums
   * (unless super.setChecksums() is abstract, i.e. this method).
   * Otherwise the lower-level checksums will not be correct.
   * 
   * @since 2.0.0
   */
  public abstract void	setChecksums();

  /**
   * Compute an IP checksum.
   *
   * Uses the calculation from RFC 1071, corrected to handle a final odd byte correctly.
   * This is (a) from a reliable source, and (b) significantly faster than the old implementation in
   * {@link IPPacket#onesCompSum}.
   * 
   * This is part of the fix for issue 1352176.
   *
   * @param bytes data
   * @param start Initial offset
   * @param len Total number of bytes to process
   * @return the ones-complement of the ones-complement sum of the data,
   * treated as big-endian 16-bit words, with any final odd byte causing an implicit
   * padding with a byte of zero.
   *
   * @since 2.0.0
   */
  protected int ipChecksum(byte[] bytes, int start, int len)
  {
	  int sum = 0;
	  // An IP checksum is the 16 bit ones-complement of the ones-complement sum of the IP header.
	  for (int i = 0; i+1 < len; i += 2)
	  {
		  sum += ((bytes[start+i] & 0xff) << 8)|(bytes[start+i+1] & 0xff);
		  // Prevent overflow into sign bit.
		  if ((sum & 0x80000000) != 0)
			  sum = (sum & 0xffff) + (sum >>> 16);
	  }
	  if ((len & 1) != 0)
	  {
		  // EJP: IN RFC 1071 this isn't shifted left 8 but it should be!
		  sum += (bytes[start+len-1] & 0xff) << 8;
	  }
	  // Propagate carries
	  while ((sum >> 16) != 0)
		  sum = (sum & 0xffff) + (sum >> 16);
	  return ~sum & 0xffff;
  }
  
  /**
   * Generate string with contents describing this packet.
   * @param colored whether or not the string should contain ansi
   * color escape sequences.
   */
  public abstract String toColoredString(boolean colored);

  private static final String _rcsid = 
    "$Id: Packet.java,v 1.3 2008/10/20 14:43:41 rickwall Exp $";
}
