/*
 * STP.java
 *
 * Created on 30 April 2008, 15:55
 *
 ***************************************************************************
 * Copyright (C) 2008, Esmond Pitt                                         *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************
 */

package net.sourceforge.jpcap.net;
import java.math.BigInteger;
import java.nio.ByteBuffer;
import net.sourceforge.jpcap.util.AnsiEscapeSequences;
import net.sourceforge.jpcap.util.Timeval;

/**
 * Ethernet 802.3 Spanning Tree Protocol (STP).
 *
 * @since 2.0.0.
 *
 * @author Esmond Pitt
 * @version $Revision: 1.1 $
 */
public interface STP extends LLC
{
	/**
	 * Offset of the version field.
	 */
	public static final int STP_VERSION_POS = LLC_HEADER_LEN;
	/**
	 * Length of the version field.
	 */
	public static final int STP_VERSION_LEN = 1;
	/**
	 * Offset of the BPDUType field.
	 */
	public static final int STP_BPDUTYPE_POS = STP_VERSION_POS+STP_VERSION_LEN;
	/**
	 * Length of the BPDUType field.
	 */
	public static final int STP_BPDUTYPE_LEN = 1;
	/**
	 * Offset of the BPDUFlags field.
	 */
	public static final int STP_BPDUFLAGS_POS = STP_BPDUTYPE_POS+STP_BPDUTYPE_LEN;
	/**
	 * Length of the BPDUFlags field.
	 */
	public static final int STP_BPDUFLAGS_LEN = 1;
	/**
	 * Offset of the Root Identifier field.
	 */
	public static final int STP_ROOTIDENT_POS = STP_BPDUFLAGS_POS+STP_BPDUFLAGS_LEN;
	/**
	 * Length of the Root Identifier field.
	 */
	public static final int STP_ROOTIDENT_LEN = 8;
	/**
	 * Offset of the Root Path Cost field.
	 */
	public static final int STP_ROOTPATHCOST_POS = STP_ROOTIDENT_POS+STP_ROOTIDENT_LEN;
	/**
	 * Length of the Root Path Cost field.
	 */
	public static final int STP_ROOTPATHCOST_LEN = 4;
	/**
	 * Offset of the Bridge Identifier field.
	 */
	public static final int STP_BRIDGEIDENT_POS = STP_ROOTPATHCOST_POS+STP_ROOTPATHCOST_LEN;
	/**
	 * Length of the Bridge Identifier field.
	 */
	public static final int STP_BRIDGEIDENT_LEN = 8;
	/**
	 * Offset of the PortIdentifier field.
	 */
	public static final int STP_PORTIDENT_POS = STP_BRIDGEIDENT_POS+STP_BRIDGEIDENT_LEN;
	/**
	 * Length of the PortIdentifier field.
	 */
	public static final int STP_PORTIDENT_LEN = 2;
	/**
	 * Offset of the MessageAge field.
	 */
	public static final int STP_MSGAGE_POS = STP_PORTIDENT_POS+STP_PORTIDENT_LEN;
	/**
	 * Length of the MessageAge field.
	 */
	public static final int STP_MSGAGE_LEN = 2;
	/**
	 * Offset of the Max Age field.
	 */
	public static final int STP_MAXAGE_POS = STP_MSGAGE_POS+STP_MSGAGE_LEN;
	/**
	 * Length of the Max Age field.
	 */
	public static final int STP_MAXAGE_LEN = 2;
	/**
	 * Offset of the HelloTime field.
	 */
	public static final int STP_HELLOTIME_POS = STP_MAXAGE_POS+STP_MAXAGE_LEN;
	/**
	 * Length of the HelloTime field.
	 */
	public static final int STP_HELLOTIME_LEN = 2;
	/**
	 * Offset of the ForwardDelay field.
	 */
	public static final int STP_FWDDELAY_POS = STP_HELLOTIME_POS+STP_HELLOTIME_LEN;
	/**
	 * Length of the ForwardDelay field.
	 */
	public static final int STP_FWDDELAY_LEN = 2;
	/**
	 * Length of the STP header.
	 */
	public static final int STP_HEADER_LEN =
		STP_VERSION_LEN
		+STP_BPDUTYPE_LEN
		+STP_BPDUFLAGS_LEN
		+STP_ROOTIDENT_LEN
		+STP_ROOTPATHCOST_LEN
		+STP_BRIDGEIDENT_LEN
		+STP_PORTIDENT_LEN
		+STP_MSGAGE_LEN
		+STP_MAXAGE_LEN
		+STP_HELLOTIME_LEN
		+STP_FWDDELAY_LEN;

	/**
	 * Encapsulates an Ethernet 802.3 Spanning Tree Protocol (STP) packet.
	 *
	 * @since 2.0.0.
	 *
	 * @author Esmond Pitt
	 * @version $Revision: 1.1 $
	 */
	public class Packet extends EthernetPacket
	{
		static final long serialVersionUID = 998466301174560929L;

		/**
		 * Create a new STP packet.
		 * @param lLen Packet length
		 * @param bytes Packet bytes
		 */
		public Packet(int lLen, byte [] bytes)
		{
			this(lLen, bytes, null);
		}
		
		/**
		 * Create a new STP packet.
		 * @param lLen Packet length
		 * @param bytes Packet bytes
		 * @param tv TimeValue
		 */
		public Packet(int lLen, byte [] bytes, Timeval tv)
		{
			this(lLen, ByteBuffer.wrap(bytes), tv);
		}
		
		/**
		 * Create a new STP packet.
		 * @param lLen Packet length
		 * @param bb Packet bytes in a ByteBuffer
		 * @param tv TimeValue
		 */
		public Packet(int lLen, ByteBuffer bb, Timeval tv)
		{
			super(lLen, bb, tv);
		}
		
		/**
		 * @return the STP protocol version
		 */
		public int	getSTPVersion()
		{
			return _byteBuffer.get(_ethOffset+STP_VERSION_POS);
		}
		
		/**
		 * Set the STP protocol version
		 * @param version STP protocol version
		 */
		public void	setSTPVersion(int version)
		{
			_byteBuffer.put(_ethOffset+STP_VERSION_POS, (byte)version);
		}
		
		/**
		 * @return the BPDU type
		 */
		public int	getSTPBPDUType()
		{
			return _byteBuffer.get(_ethOffset+STP_BPDUTYPE_POS);
		}
		
		/**
		 * Set the BPDU type
		 * @param type BPDU type
		 */
		public void	setSTPBPDUType(int type)
		{
			_byteBuffer.put(_ethOffset+STP_BPDUTYPE_POS, (byte)type);
		}
		
		/**
		 * @return the BPDU flags
		 */
		public int	getSTPBPDUFlags()
		{
			return _byteBuffer.get(_ethOffset+STP_BPDUFLAGS_POS);
		}
		
		/**
		 * Set the BPDU flags
		 * @param flags BPDU flags
		 */
		public void	setSTPBPDUFlags(int flags)
		{
			_byteBuffer.put(_ethOffset+STP_BPDUFLAGS_POS, (byte)flags);
		}
		
		/**
		 * @return the Root Identifier
		 */
		public byte[]	getSTPRootIdentifier()
		{
			byte[]	rootIdent = new byte[STP_ROOTIDENT_LEN];
			_byteBuffer.position(_ethOffset+STP_ROOTIDENT_POS);
			_byteBuffer.get(rootIdent);
			return rootIdent;
		}
		
		/**
		 * Set the RootIdentifier
		 * @param rootIdent RootIdentifier
		 */
		public void	setSTPRootIdentifier(byte[] rootIdent)
		{
			assert rootIdent.length == STP_ROOTIDENT_LEN : "STPRootIdentifier must be "+STP_ROOTIDENT_LEN+" bytes";
			_byteBuffer.position(_ethOffset+STP_ROOTIDENT_POS);
			_byteBuffer.put(rootIdent);
		}
		
		/**
		 * @return the Root path cost
		 */
		public int	getSTPRootPathCost()
		{
			return _byteBuffer.getShort(_ethOffset+STP_ROOTPATHCOST_POS) & 0xffff;
		}
		
		/**
		 * Set the Root Path Cost
		 * @param cost Root Path Cost
		 */
		public void	setSTPRootPathCost(int cost)
		{
			_byteBuffer.putShort(_ethOffset+STP_ROOTPATHCOST_POS, (short)cost);
		}
		
		/**
		 * @return the Bridge Identifier
		 */
		public byte[]	getSTPBridgeIdentifier()
		{
			byte[]	bridgeIdent = new byte[STP_BRIDGEIDENT_LEN];
			_byteBuffer.position(_ethOffset+STP_BRIDGEIDENT_POS);
			_byteBuffer.get(bridgeIdent);
			return bridgeIdent;
		}
		
		/**
		 * Set the Bridge Identifier
		 * @param bridgeIdent Bridge Identifier
		 */
		public void	setSTPBridgeIdentifier(byte[] bridgeIdent)
		{
			assert bridgeIdent.length == STP_BRIDGEIDENT_LEN : "STPBridgeIdentifier must be "+STP_BRIDGEIDENT_LEN+" bytes";
			_byteBuffer.position(_ethOffset+STP_BRIDGEIDENT_POS);
			_byteBuffer.put(bridgeIdent);
		}
		
		/**
		 * @return the Port identifier
		 */
		public int	getSTPPortIdentifier()
		{
			return _byteBuffer.getShort(_ethOffset+STP_PORTIDENT_POS) & 0xffff;
		}
		
		/**
		 * Set the Port Identifier
		 * @param id Port Identifier
		 */
		public void	setSTPPortIdentifier(int id)
		{
			_byteBuffer.putShort(_ethOffset+STP_PORTIDENT_POS, (short)id);
		}
		
		/**
		 * @return the Message Age
		 */
		public int	getSTPMessageAge()
		{
			return _byteBuffer.getShort(_ethOffset+STP_MSGAGE_POS);
		}
		
		/**
		 * Set the Message Age
		 * @param age Message Age
		 */
		public void	setSTPMessageAge(int age)
		{
			_byteBuffer.putShort(_ethOffset+STP_MSGAGE_POS, (short)age);
		}
		
		/**
		 * @return the maximum age
		 */
		public int	getSTPMaxAge()
		{
			return _byteBuffer.getShort(_ethOffset+STP_MAXAGE_POS);
		}
		
		/**
		 * Set the maximum age
		 * @param maxAge Maximum age
		 */
		public void	setSTPMaxAge(int maxAge)
		{
			_byteBuffer.putShort(_ethOffset+STP_MAXAGE_POS, (short)maxAge);
		}
		
		/**
		 * @return the Hello time
		 */
		public int	getSTPHelloTime()
		{
			return _byteBuffer.getShort(_ethOffset+STP_HELLOTIME_POS);
		}
		
		/**
		 * Set the HelloTime
		 * @param time Time
		 */
		public void	setSTPHelloTime(int time)
		{
			_byteBuffer.putShort(_ethOffset+STP_HELLOTIME_POS, (short)time);
		}
		
		/**
		 * @return the forward delay
		 */
		public int	getSTPForwardDelay()
		{
			return _byteBuffer.getShort(_ethOffset+STP_FWDDELAY_POS);
		}
		
		/**
		 * Set the Forward Delay
		 *
		 * @param delay Delay
		 */
		public void	setSTPForwardDelay(int delay)
		{
			_byteBuffer.putShort(_ethOffset+STP_FWDDELAY_POS, (short)delay);
		}
		
		public String	toColoredString(boolean colored)
		{
			StringBuffer buffer = new StringBuffer();
			buffer.append('[');
			if(colored) buffer.append(getColor());
			buffer.append("STPPacket");
			if(colored) buffer.append(AnsiEscapeSequences.RESET);
			buffer.append(": ");
			buffer.append(getSourceMACAddress().toString() + "->" +
				getDestinationMACAddress().toString());
			byte[]	root = getSTPRootIdentifier();
			int	id = ByteBuffer.wrap(root).getShort() & 0xffff;
			String	mac = MACAddress.extract(2,  root);
			buffer.append(" root="+id+"/"+mac);
			buffer.append(" cost="+getSTPRootPathCost());
			buffer.append(" bridge="+new BigInteger(1, getSTPBridgeIdentifier()).toString(16));
			// etc ...
			buffer.append(" l="+STP_HEADER_LEN);
			buffer.append(']');
			return buffer.toString();
			
		}
	}
}
