// $Id: PacketCaptureSimulator.java,v 1.2 2008/08/18 05:34:38 esmondpitt Exp $

/***************************************************************************
 * Copyright (C) 2001, Patrick Charles and Jonas Lehmann                   *
 * Distributed under the Mozilla Public License                            *
 *   http://www.mozilla.org/NPL/MPL-1.1.txt                                *
 ***************************************************************************/
package net.sourceforge.jpcap.simulator;
import java.nio.ByteBuffer;
import net.sourceforge.jpcap.net.LinkLayers;
import net.sourceforge.jpcap.capture.InvalidFilterException;
import net.sourceforge.jpcap.capture.PacketCaptureBase;
import net.sourceforge.jpcap.capture.PacketCaptureCapable;
import net.sourceforge.jpcap.capture.CaptureStatistics;
import net.sourceforge.jpcap.capture.CapturePacketException;
import net.sourceforge.jpcap.capture.CaptureConfigurationException;
import net.sourceforge.jpcap.capture.CaptureDevice;
import net.sourceforge.jpcap.capture.CaptureDeviceOpenException;
import net.sourceforge.jpcap.capture.CaptureDeviceNotFoundException;


/**
 * This class has the same external interface as PacketCapture.
 * <p>
 * Instead of capturing and dispatching packets from a physical network
 * device, however, this class generates fake packets.
 * The type and frequency of the packets is set by policies configured in
 * the simulator property file.
 * <p>
 * Instances of this class are used mostly by developers working on
 * applications that utilize jpcap in environments where a real 
 * network device isn't available or when the type of packets arriving 
 * needs to be carefully controlled.
 * <p>
 * For more documentation on this class's methods, see PacketCaptureCapable;
 * Javadoc is 'inherited' from this interface.
 *
 * @author Patrick Charles and Jonas Lehmann
 * @author Esmond Pitt
 * @version $Revision: 1.2 $
 * @lastModifiedBy $Author: esmondpitt $
 * @lastModifiedAt $Date: 2008/08/18 05:34:38 $
 */
public class PacketCaptureSimulator extends PacketCaptureBase
  implements PacketCaptureCapable
{
	private static final String	Device = "jpcapsim0";
	
	private int	receivedCount,
				droppedCount;
	
  public PacketCaptureSimulator() {
    // the simulator currently only simulates ethernet with link-level headers
    linkType = LinkLayers.EN10MB;
  }

  // methods for controlling a (simulated) packet capture session

  public void open(String device, boolean promiscuous)
   throws CaptureDeviceOpenException {
    open(device, DEFAULT_SNAPLEN, promiscuous, DEFAULT_TIMEOUT);
  }

  public void open(String device, int snaplen, 
                   boolean promiscuous, int timeout) {
    // noop
  }

  public void openOffline(String fileName) {
    // noop
  }

  public void setFilter(String filterExpression, boolean optimize)
    throws InvalidFilterException {
    // noop. the simulator could potentially parse the filter and 
    // modify the packet generator's engine accordingly. 
    // instead, though, prefer to control the generator's behavior
    // via the simulator properties. the complexities of bpf are 
    // currently beyond the scope of the simulator. :/
 
	  // EJP NOTE: this could now use isFilteredOffline(), with a bit of jiggery-pokery ...
  }

  /**
   * {@inheritDoc}
   * The simulator's implementation of capture causes packets to be 
   * randomly generated. The packet frequency and type are configurable.
   */
  public void capture(int count)
    throws CapturePacketException {
    for(int i=0; i<count && capturing; i++) {
      byte [] bytes = PacketGenerator.generate();

      long millis = System.currentTimeMillis();
      int seconds = (int)(millis / 1000);
      int useconds = (int)((millis % 1000)*1000);

      super.handlePacket(bytes.length, bytes.length, seconds, useconds, ByteBuffer.wrap(bytes));
      receivedCount ++;
    }
  }
  
  private boolean	capturing = true;
  
  /**
   * {@inheritDoc}
   */
  public void	endCapture()
  {
	this.capturing = false;
  }

  public CaptureStatistics getStatistics() {
    return new CaptureStatistics(receivedCount, droppedCount);
  }

  public void close() {
    // noop
  }

  // static native methods to fetch capture device and network information

  public String findDevice()
  {
    return Device;
  }

  /**
   * TODO this is static in {@link net.sourceforge.jpcap.capture.PacketCapture}.
   */
  public String[] lookupDevices()
		throws CaptureDeviceNotFoundException
  {
    return new String [] {findDevice()};
  }

  /**
   * @see net.sourceforge.jpcap.capture.PacketCapture#findCaptureDevice
   */
  public static CaptureDevice	findCaptureDevice()
  {
	  return lookupCaptureDevices()[0];
  }
  
  /**
   * @see net.sourceforge.jpcap.capture.PacketCapture#lookupCaptureDevices
   */
  public static CaptureDevice[]	lookupCaptureDevices()
  {
	  return new CaptureDevice[]
	  {
		new SimulatorCaptureDevice(Device, "Simulator capture device", 0x7f000001, 0, 0, new CaptureDevice.Address[0])
	  };
  }
  
  static class SimulatorCaptureDevice extends CaptureDevice
  {
	  	SimulatorCaptureDevice (String name, String description, int network, int netmask, int flags, Address[] addresses)
		{
			super(name, description, network, netmask, flags, addresses);
		}
  }

  public int getNetwork(String device)
    throws CaptureConfigurationException {
    return 0x00000000;
  }

  public int getNetmask(String device)
    throws CaptureConfigurationException {
    return 0x00000000;
  }

  public String	getLinkLayerDescription()
  {
	return "Ethernet";
  }
  
  public int getLinkLayerType()
    throws CaptureConfigurationException {
    return linkType;
  }

  public int getSnapshotLength() {
    return DEFAULT_SNAPLEN;
  }

  private static final String _rcsid = 
    "$Id: PacketCaptureSimulator.java,v 1.2 2008/08/18 05:34:38 esmondpitt Exp $";
}
